#!/usr/bin/perl

### ----------------------------------------------- ###
#  datatypes.pl
#  -----------
#
#  input: datatypes.txt
#
#  input format: 
#  (DataStructures.data_structure<tab>DataStructureComponents.lfd_nr<tab>
#  DataStructures.description<tab>DataStructureComponents.table_id<tab>
#  Components.description<tab>Components.table_id<tab>Components.data_type_code)+
#
#  output: datatypes.dtd
#
#
#  (last modified August 11, 1999)
### ----------------------------------------------- ###

open (infile, "datatypes.txt");
open (outfile, ">datatypes.dtd");

$eldef = "";
$prior_dt = "AD";
$definition = "";

while ($line = <infile>) {
  chop ($line);

# parse components
  @row = split(/\t/,$line);
  $dt = $row[0];
  $seq = $row[1];
  $dt_name = $row[2];
  $table1 = $row[3];
  $component_name = $row[4];
  $table2 = $row[5];
  $component_dt = $row[6];

$dt =~ s/\+/\_/g;
$component_name =~ s/\&/&amp;/g;
$table = $table2;
if ($table1 ne "0") {$table = $table1}
$component = $component_name."^".$component_dt."^".$table;
if ($seq eq "") {$component = ""}

if ($dt eq $prior_dt) {
  $definition .= $component."\t"}
else {
  &build_definitions;
  $prior_dt = $dt;
  $definition = $component."\t"}
} 

&build_definitions;
print outfile "\n\n$eldef";

# close files
close (infile);
close (outfile);

sub build_definitions {
  if ($definition eq "\t") {       # primitive data types
    if ($prior_dt eq "WILDCARD") {print outfile "<!ENTITY % WILDCARD \"(ANY)\">\n"} 
    else {print outfile "<!ENTITY % $prior_dt \"(#PCDATA)\">\n"}
    }
  else {                         # composite data types
    @definition = split(/\t/,$definition);
    $counter = 1;
    $newdefin = "";
    foreach $i (@definition) {
      @component = split(/\^/,$i);
      $name = $component[0];
      $type = $component [1];
      $tab = $component [2];

#      $i =~ s/.*\((.*)\)\s*\>\s*/\1/;
#      $name =~ s/.*\<\s*(\S.*\S)\s*\(.*/\1/;

      $newdefin .= ",".$prior_dt.".".$counter."?";

      $eldef .= "<!ENTITY % $prior_dt.$counter.CONTENT \"%$type;\">\n";
      $eldef .= "<!ELEMENT $prior_dt.$counter %$prior_dt.$counter.CONTENT;>\n";
      $eldef .= "<!ENTITY % $prior_dt.$counter.ATTRIBUTES\n  \"LongName CDATA #FIXED \'$name\'\n  Type CDATA #FIXED \'$type\'\n  Table CDATA #FIXED \'$tab\'\">\n";

      $eldef .= "<!ATTLIST $prior_dt.$counter %$prior_dt.$counter.ATTRIBUTES;>\n";
      $counter++;
      }
    $newdefin =~ s/^,//;
    print outfile "<!ENTITY % $prior_dt \"($newdefin)\">\n";
    }
}