#!/usr/bin/perl

### ----------------------------------------------- ###
#  messages.pl
#  -----------
#
#  input: messages.txt
#
#  input format: 
#  (message_structure<tab>lfd_nr<tab>seg_code<tab>
#   repetitional<tab>optional<cr>)+
#
#  output: messages.dtd
#
#
#  (last revised August 11, 1999)
### ----------------------------------------------- ###

open (infile, "messages.txt");
open (outfile, ">messages.dtd");

$prior_message = "ACK";
$definition = "";
$message = ""; 
$lfd_nr = "";
$seg_code = "";

while ($line = <infile>) {
  
  $list = 1; $lststr = "";
  $group = 1;
  $position = 0;
  $status = 0;
  $char = "";
  $counter = 0;

  chop($line);

  # parse message and definition component
  @row = split(/\t/,$line);
  $message = $row[0];
  $lfd_nr = $row[1];
  $seg_code = $row[2];
  $rep = $row[3];
  $opt = $row[4];

if ($seg_code eq "?") {$seg_code = "OTHER"}

if ($rep eq "1") {$seg_code = "{".$seg_code."}"}
if ($opt eq "1") {$seg_code = "[".$seg_code."]"}

if ($message eq $prior_message) {
  $definition .= $seg_code." "}
else {
  &add_commas;
  &remove_whitespace;
  &convert_occ_ind;
  &merge_occ_ind;
  &deepen_hierarchy;
  &print_XML_definitions;  
  $prior_message = $message;
  $definition = $seg_code." "}
}

&add_commas;
&remove_whitespace;
&convert_occ_ind;
&merge_occ_ind;
&deepen_hierarchy;
&print_XML_definitions;  

# close files
close (infile);
close (outfile);


sub add_commas {
  $definition =~ s/(\w)([\s\}\]]*)\s([\w[{])/\1\2,\3/g;
}

sub remove_whitespace {
  $definition =~ s/\s//g;
}

sub convert_occ_ind {
  # convert brackets and braces to parenthesis and occurrence indicators
  $definition =~ s/[{[]/(/g;
  $definition =~ s/}/)+/g;
  $definition =~ s/]/)?/g;
}

sub merge_occ_ind {
  # merge "?" and "+" occurrence indicators where possible
  $definition =~ s/\(\((\w+)\)\?\)\+/\(\1\)\*/g;
  $definition =~ s/\(\((\w+)\)\+\)\?/\(\1\)\*/g;
}

sub deepen_hierarchy {
  # deepen the XML element hierarchy
  while ($definition =~ /[\*\+]/) {
    $status = 0;
    $posS = index($definition,"*");
    $posP = index($definition,"+");
    $position=$posP; $char="+";
    if ($posP < 0) {$position=$posS; $char="*"}
    if ($posS < $posP) {$position=$posS; $char="*"}
    if ($posS < 0) {$position=$posP; $char="+"}

    for ($counter = $position-2; $counter > -1; $counter--) {
      if (substr($definition, $counter, 1) eq ")") {$status++}
      if (substr($definition, $counter, 1) eq "(") {
        $status--;
        if ($status < 0) {
          $lststr = substr($definition,$counter+1,($position-$counter)-2);
          $lststr =~ s/\(([\w\.]+)\)/\1/g;
          if ($lststr =~ /\,/) {
            &remove_subgroups;
            print outfile "<!ENTITY % $prior_message.GRP.$group.CONTENT \"($lststr)\">\n";
            print outfile "<!ELEMENT $prior_message.GRP.$group %$prior_message.GRP.$group.CONTENT;>\n";
            $lststr = $prior_message.".GRP.".$group;
            $group++;
            }
          if ($char eq "+") {
            substr($definition,$counter,($position-$counter)+1) = $prior_message.".LST.".$list;
            print outfile "<!ENTITY % $prior_message.LST.$list.CONTENT \"($lststr)+\">\n";
            print outfile "<!ELEMENT $prior_message.LST.$list %$prior_message.LST.$list.CONTENT;>\n";
            $list++;
            }
          if ($char eq "*") {
            substr($definition,$counter,($position-$counter)+1) = $prior_message.".LST.".$list."?";
            print outfile "<!ENTITY % $prior_message.LST.$list.CONTENT \"($lststr)+\">\n";
            print outfile "<!ELEMENT $prior_message.LST.$list %$prior_message.LST.$list.CONTENT;>\n";
            $list++;
            }
          $counter = 0;
          }
        }
      }
  }
}

sub print_XML_definitions {
  $definition =~ s/\(([\w\.]+)\)/\1/g;
  print outfile "<!ENTITY % $prior_message.CONTENT \"($definition)\">\n";
  print outfile "<!ELEMENT $prior_message %$prior_message.CONTENT;>\n\n";
}

sub remove_subgroups {
  while (($lststr =~ /\)\?/) && ($group < 15)) {
    $tmpstr = $prior_message.".GRP.".$group;
    $lststr =~ s/\(([\w\.\,\?]+)\)\?/$tmpstr\?/;
    print outfile "<!ENTITY % $prior_message.GRP.$group.CONTENT \"$1\">\n";
    print outfile "<!ELEMENT $prior_message.GRP.$group (%$prior_message.GRP.$group.CONTENT;)>\n";
    $group++;
  } 
}
